%  k-means method for trends (KMT) --- multi factors time series
%
%  by Norio Watanabe, Chuo University, JAPAN
%                                         2/20/2023
%                                        11/12/2022
%
%  Reference:
%
%     A generalization of the k-means method for trends of time series
%          ASMDA 2023 (20th Applied Stochastic Models and Data Analysis
%          International Conference)
%
%  contents:
%     KMTm1.m : main M-file using following functions
%            kmeansTrendM.m
%            CommonTrendM.m
%            fun_dsmCTm.m
%            fun_dsmTm.m
%     KMTm2.m : auxiliary program (sort and summarize)
%
clear
close all
%%%%%%%%%%%%%%%%%%
K=2;       % number of clusters ( several values of K should be tried )
Lmax=20;   % max lag (depends on time series. 0 is also accepted)
weight=[1 1 1]'; % weight for dissimilarity measure
%%%%%%%%%%%%%%%%%%
JCmax= 20; % iteration number
           % set large, if not well classified becuase of sensitivity on initial values
rand('seed',87) % for setting initial values (old style)
%%%%%%%%%%%%%%%%%%
fname='covid19TK';
           % time series on COVID-19 in JAPAN (P=47, Q=3, see Reference)
           %    PCR COVID-19 tested positives
           %    COVID-19 serious cases
           %    COVID-19 deaths
           % The data set is provided by Toyo Keizai Online 
           % "Coronavirus Disease (COVID-19) Situation Report in Japan"
           %
           %    Xori : original time series
           %    X    : moving averaged series
           %
load(strcat(fname,'.mat'));
%%%%%%%%%%%%%%%%%%
%
% X: P x Q x N  matrix
%    P : number of objects (each object consists of Q-variate time series)
%    N :length of series
%
[P,Q,N]=size(X);
if length(weight)<Q
    weight=ones(Q,1);
end
%%%%%%%%%%%%%%%%%%
[CMT,kc,Lag,R,Jmin]=kmeansTrendM(X,K,JCmax,Lmax,weight);
%  X  : PxQxN
%  CMT: KxQxNT  R:PxQ
%%%%%%%%%%%%%%%%%%

NT=N-2*Lmax;
for k=1:K
  for q=1:Q
    subplot(K,Q,(k-1)*Q+q)
    hold on
    for p=1:P
        n1=Lmax+1+Lag(p);
        n2=N-Lmax+Lag(p);
        if kc(p)==k
            y=squeeze(X(p,q,:));
            y=y(n1:n2);
            plot([N-n2+1:N-n1+1],R(p,q)*y,'b:')
        end
    end
    plot([Lmax+1:N-Lmax],squeeze(CMT(k,q,:)),'r-','LineWidth',3)
    hold off
    box on
  end
end
for k=1:K
    sum(kc==k)
end

Jmin

% save results ( used by KMTm2.m )

save(strcat(fname,'_res'),'K','Lmax','Jmin','kc','Lag','R','CMT') 


